'use strict'

Object.defineProperty(exports, '__esModule', { value: true })

function _interopDefault(ex) {
  return ex && typeof ex === 'object' && 'default' in ex ? ex['default'] : ex
}

var path = require('path')
var compositionApi = require('@vue/composition-api')
var defu = _interopDefault(require('defu'))
var Vue = _interopDefault(require('vue'))

function getValue(value) {
  if (value instanceof Function) return value()
  return value
}
let data = {}
function setSSRContext(ssrContext) {
  data = Object.assign({}, {})
  ssrContext.nuxt.ssrRefs = data
}
const isProxyable = val => val && typeof val === 'object'
/**
 * Creates a Ref that is in sync with the client.
 */
const ssrRef = (value, key) => {
  var _a, _b, _c
  if (!key) {
    throw new Error(
      "You must provide a key. You can have it generated automatically by adding 'nuxt-composition-api/babel' to your Babel plugins."
    )
  }
  if (process.client) {
    return compositionApi.ref(
      (_c =
        (_b =
          (_a = window.__NUXT__) === null || _a === void 0
            ? void 0
            : _a.ssrRefs) === null || _b === void 0
          ? void 0
          : _b[key]) !== null && _c !== void 0
        ? _c
        : getValue(value)
    )
  }
  const val = getValue(value)
  const _ref = compositionApi.ref(val)
  if (value instanceof Function) data[key] = val
  const getProxy = observable =>
    new Proxy(observable, {
      get(target, prop) {
        if (isProxyable(target[prop])) return getProxy(target[prop])
        return Reflect.get(target, prop)
      },
      set(obj, prop, val) {
        data[key] = _ref.value
        return Reflect.set(obj, prop, val)
      },
    })
  const proxy = compositionApi.computed({
    get: () => (isProxyable(_ref.value) ? getProxy(_ref.value) : _ref.value),
    set: v => {
      data[key] = v
      _ref.value = v
    },
  })
  return proxy
}

const useAsync = (cb, key) => {
  if (!key) {
    throw new Error(
      "You must provide a key. You can have it generated automatically by adding 'nuxt-composition-api/babel' to your Babel plugins."
    )
  }
  const _ref = compositionApi.isRef(key) ? key : ssrRef(null, key)
  if (!_ref.value) {
    const p = cb()
    if (p instanceof Promise) {
      if (process.server) {
        compositionApi.onServerPrefetch(async () => {
          _ref.value = await p
        })
      } else {
        // eslint-disable-next-line
        p.then(res => (_ref.value = res))
      }
    } else {
      _ref.value = p
    }
  }
  return _ref
}

function createEmptyMeta() {
  return {
    __dangerouslyDisableSanitizers: [],
    __dangerouslyDisableSanitizersByTagID: {},
    title: undefined,
    titleTemplate: undefined,
    htmlAttrs: {},
    headAttrs: {},
    bodyAttrs: {},
    base: undefined,
    meta: [],
    link: [],
    style: [],
    script: [],
    noscript: [],
    changed: undefined,
    afterNavigation: undefined,
  }
}

const defineComponent = options => {
  if (!('head' in options)) return options
  const _head = compositionApi.reactive({})
  if (!(options.head instanceof Function)) {
    Object.assign(_head, options.head)
  }
  const head =
    options.head instanceof Function
      ? () => defu(_head, options.head())
      : () => _head
  return {
    ...options,
    _head,
    head,
  }
}
const useMeta = init => {
  const vm = compositionApi.getCurrentInstance()
  if (!vm) throw new Error('useMeta must be called within a component.')
  if (!('_head' in vm.$options))
    throw new Error(
      'In order to enable `useMeta`, please make sure you include `head: {}` within your component definition, and you are using the `defineComponent` exported from nuxt-composition-api.'
    )
  const { _head } = vm.$options
  Object.assign(_head, createEmptyMeta())
  Object.assign(_head, init || {})
  return compositionApi.toRefs(_head)
}

/**
 * @deprecated
 * Recommend using `useContext` instead
 */
const withContext = callback => {
  const vm = compositionApi.getCurrentInstance()
  if (!vm) throw new Error('This must be called within a setup function.')
  callback(vm.$nuxt.context)
}
const useContext = () => {
  const vm = compositionApi.getCurrentInstance()
  if (!vm) throw new Error('This must be called within a setup function.')
  return vm.$nuxt.context
}

function normalizeError(err) {
  let message
  if (!(err.message || typeof err === 'string')) {
    try {
      message = JSON.stringify(err, null, 2)
    } catch (e) {
      message = `[${err.constructor.name}]`
    }
  } else {
    message = err.message || err
  }
  return {
    ...err,
    message,
    statusCode:
      err.statusCode ||
      err.status ||
      (err.response && err.response.status) ||
      500,
  }
}
const fetches = new WeakMap()
const isSsrHydration = vm => {
  var _a, _b, _c
  return (_c =
    (_b = (_a = vm.$vnode) === null || _a === void 0 ? void 0 : _a.elm) ===
      null || _b === void 0
      ? void 0
      : _b.dataset) === null || _c === void 0
    ? void 0
    : _c.fetchKey
}
const nuxtState = process.client && window.__NUXT__
function registerCallback(vm, callback) {
  const callbacks = fetches.get(vm) || []
  fetches.set(vm, [...callbacks, callback])
}
async function callFetches() {
  const fetchesToCall = fetches.get(this)
  if (!fetchesToCall) return
  this.$nuxt.nbFetching++
  this.$fetchState.pending = true
  this.$fetchState.error = null
  this._hydrated = false
  let error = null
  const startTime = Date.now()
  try {
    await Promise.all(fetchesToCall.map(fetch => fetch(this)))
  } catch (err) {
    error = normalizeError(err)
  }
  const delayLeft = (this._fetchDelay || 0) - (Date.now() - startTime)
  if (delayLeft > 0) {
    await new Promise(resolve => setTimeout(resolve, delayLeft))
  }
  this.$fetchState.error = error
  this.$fetchState.pending = false
  this.$fetchState.timestamp = Date.now()
  this.$nextTick(() => this.$nuxt.nbFetching--)
}
async function serverPrefetch(vm) {
  if (!vm._fetchOnServer) {
    return
  }
  // Call and await on $fetch
  vm.$fetchState =
    vm.$fetchState ||
    Vue.observable({
      error: null,
      pending: false,
      timestamp: 0,
    })
  try {
    await callFetches.call(vm)
  } catch (err) {
    vm.$fetchState.error = normalizeError(err)
  }
  vm.$fetchState.pending = false
  // Define an ssrKey for hydration
  vm._fetchKey = vm.$ssrContext.nuxt.fetch.length
  // Add data-fetch-key on parent element of Component
  if (!vm.$vnode.data) vm.$vnode.data = {}
  const attrs = (vm.$vnode.data.attrs = vm.$vnode.data.attrs || {})
  attrs['data-fetch-key'] = vm._fetchKey
  // Add to ssrContext for window.__NUXT__.fetch
  vm.$ssrContext.nuxt.fetch.push(
    vm.$fetchState.error
      ? { _error: vm.$fetchState.error }
      : JSON.parse(JSON.stringify(vm._data))
  )
}
const useFetch = callback => {
  var _a
  const vm = compositionApi.getCurrentInstance()
  if (!vm) throw new Error('This must be called within a setup function.')
  vm.$fetchState =
    vm.$fetchState ||
    Vue.observable({
      error: null,
      pending: false,
      timestamp: 0,
    })
  vm.$fetch = callFetches.bind(vm)
  registerCallback(vm, callback)
  if (typeof vm.$options.fetchOnServer === 'function') {
    vm._fetchOnServer = vm.$options.fetchOnServer.call(vm) !== false
  } else {
    vm._fetchOnServer = vm.$options.fetchOnServer !== false
  }
  compositionApi.onServerPrefetch(() => serverPrefetch(vm))
  compositionApi.onBeforeMount(() => {
    if (!vm._hydrated) {
      return callFetches.call(vm)
    }
  })
  if (process.server || !isSsrHydration(vm))
    return {
      $fetch: vm.$fetch,
      $fetchState: vm.$fetchState,
    }
  // Hydrate component
  vm._hydrated = true
  vm._fetchKey = +((_a = vm.$vnode.elm) === null || _a === void 0
    ? void 0
    : _a.dataset.fetchKey)
  const data = nuxtState.fetch[vm._fetchKey]
  // If fetch error
  if (data && data._error) {
    vm.$fetchState.error = data._error
    return {
      $fetch: vm.$fetch,
      $fetchState: vm.$fetchState,
    }
  }
  compositionApi.onBeforeMount(() => {
    // Merge data
    for (const key in data) {
      try {
        Vue.set(vm, key, data[key])
      } catch (e) {
        if (process.env.NODE_ENV === 'development')
          // eslint-disable-next-line
          console.warn(`Could not hydrate ${key}.`)
      }
    }
  })
  return {
    $fetch: vm.$fetch,
    $fetchState: vm.$fetchState,
  }
}

const compositionApiModule = function () {
  const libRoot = path.resolve(__dirname, '..')
  const { dst } = this.addTemplate({
    src: path.resolve(libRoot, 'lib', 'plugin.js'),
    fileName: path.join('composition-api', 'plugin.js'),
    options: {},
  })
  this.options.build = this.options.build || {}
  this.options.build.babel = this.options.build.babel || {}
  this.options.build.babel.plugins = this.options.build.babel.plugins || []
  this.options.build.babel.plugins.push(path.join(__dirname, 'babel'))
  this.options.build.transpile = this.options.build.transpile || []
  this.options.build.transpile.push(/nuxt-composition-api/)
  this.options.plugins = this.options.plugins || []
  this.options.plugins.push(path.resolve(this.options.buildDir || '', dst))
}
// eslint-disable-next-line
const meta = require('../package.json')

Object.defineProperty(exports, 'computed', {
  enumerable: true,
  get: function () {
    return compositionApi.computed
  },
})
Object.defineProperty(exports, 'createComponent', {
  enumerable: true,
  get: function () {
    return compositionApi.createComponent
  },
})
Object.defineProperty(exports, 'createElement', {
  enumerable: true,
  get: function () {
    return compositionApi.createElement
  },
})
Object.defineProperty(exports, 'getCurrentInstance', {
  enumerable: true,
  get: function () {
    return compositionApi.getCurrentInstance
  },
})
Object.defineProperty(exports, 'inject', {
  enumerable: true,
  get: function () {
    return compositionApi.inject
  },
})
Object.defineProperty(exports, 'isRef', {
  enumerable: true,
  get: function () {
    return compositionApi.isRef
  },
})
Object.defineProperty(exports, 'onActivated', {
  enumerable: true,
  get: function () {
    return compositionApi.onActivated
  },
})
Object.defineProperty(exports, 'onBeforeMount', {
  enumerable: true,
  get: function () {
    return compositionApi.onBeforeMount
  },
})
Object.defineProperty(exports, 'onBeforeUnmount', {
  enumerable: true,
  get: function () {
    return compositionApi.onBeforeUnmount
  },
})
Object.defineProperty(exports, 'onBeforeUpdate', {
  enumerable: true,
  get: function () {
    return compositionApi.onBeforeUpdate
  },
})
Object.defineProperty(exports, 'onDeactivated', {
  enumerable: true,
  get: function () {
    return compositionApi.onDeactivated
  },
})
Object.defineProperty(exports, 'onErrorCaptured', {
  enumerable: true,
  get: function () {
    return compositionApi.onErrorCaptured
  },
})
Object.defineProperty(exports, 'onMounted', {
  enumerable: true,
  get: function () {
    return compositionApi.onMounted
  },
})
Object.defineProperty(exports, 'onServerPrefetch', {
  enumerable: true,
  get: function () {
    return compositionApi.onServerPrefetch
  },
})
Object.defineProperty(exports, 'onUnmounted', {
  enumerable: true,
  get: function () {
    return compositionApi.onUnmounted
  },
})
Object.defineProperty(exports, 'onUpdated', {
  enumerable: true,
  get: function () {
    return compositionApi.onUpdated
  },
})
Object.defineProperty(exports, 'provide', {
  enumerable: true,
  get: function () {
    return compositionApi.provide
  },
})
Object.defineProperty(exports, 'reactive', {
  enumerable: true,
  get: function () {
    return compositionApi.reactive
  },
})
Object.defineProperty(exports, 'ref', {
  enumerable: true,
  get: function () {
    return compositionApi.ref
  },
})
Object.defineProperty(exports, 'set', {
  enumerable: true,
  get: function () {
    return compositionApi.set
  },
})
Object.defineProperty(exports, 'toRefs', {
  enumerable: true,
  get: function () {
    return compositionApi.toRefs
  },
})
Object.defineProperty(exports, 'watch', {
  enumerable: true,
  get: function () {
    return compositionApi.watch
  },
})
Object.defineProperty(exports, 'watchEffect', {
  enumerable: true,
  get: function () {
    return compositionApi.watchEffect
  },
})
exports.default = compositionApiModule
exports.defineComponent = defineComponent
exports.meta = meta
exports.setSSRContext = setSSRContext
exports.ssrRef = ssrRef
exports.useAsync = useAsync
exports.useContext = useContext
exports.useFetch = useFetch
exports.useMeta = useMeta
exports.withContext = withContext
